@@ -21,9 +21,11 @@ import (
 	"bufio"
 	"bytes"
 	"encoding/hex"
+	"errors"
 	"flag"
 	"fmt"
 	"io"
+	"io/fs"
 	"log"
 	"os"
 	"os/signal"
@@ -392,7 +394,12 @@ func main() {
 		var dos []string
 		dos, err = whichdo(tgts[0])
 		if err != nil {
-			break
+			if errors.Is(err, fs.ErrNotExist) {
+				err = nil
+				ok = false
+			} else {
+				break
+			}
 		}
 		for _, do := range dos {
 			fmt.Println(do)
